   
; *******************************************************************
; EDU10 Lesson 6 - Switch Debounce
;
; This shows one method to debounce switches.  
; SW1 is debounced
; The LEDs are updated with a
; count incremented on each switch SW1 press.
; *******************************************************************
; * For more details see Microchip's 44-pin Demo Board User's Guide *
; * Chapter 3. 44-Pin Demo Board Lessons                            *
; * http://ww1.microchip.com/downloads/en/DeviceDoc/41296B.pdf      *
; *******************************************************************

#include <p16F882.inc>

	__CONFIG    _CONFIG1, _LVP_OFF & _FCMEN_OFF & _IESO_OFF & _BOR_OFF & _CPD_OFF & _CP_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT
	__CONFIG    _CONFIG2, _WRT_OFF & _BOR21V

	cblock     0x20
previous_state
old_counter
debounce
button_pressed
button_released
counter
tmp
    endc
     
    org 0x00
	goto	Start
Start:
	bsf     STATUS,RP0       		; select Register Bank 1
	movlw   B'11111111'
	movwf   TRISA            		; Make PORTA all input
	movlw   B'00001111'
	movwf   TRISC               	; Make PORTC RC4-RC7 as outputs
	bsf		OSCCON,IRCF0			; 8MHz osc.
	bcf		STATUS,RP0     			; back to Register Bank 0          	
	clrf	PORTC            		; PortC LEDs all off
	clrf    counter
	clrf	previous_state
	movlw   0xff					; debounce time
	movwf	debounce				; store it
	movwf	button_pressed			; set the button pressed debounce time counter
	movwf	button_released			; set the button released debounce time counter

MainLoop:
	call	is_sw1_released
	call	is_sw1_pressed
	call	inc_counter
	movf	counter,w
	movwf	tmp
	swapf	tmp,w					; Swap nibbles
	movwf	PORTC					; show on LEDs the low nibble of the button press counter 
	goto    MainLoop

is_sw1_released:
	clrw
	iorwf	button_pressed,w	
	btfss	STATUS,Z			; is button pressed time complete 
	return						; no -- wait for the button pressed time to complete first
	clrw
	iorwf	button_released,w	
	btfsc	STATUS,Z			; is button released time complete 
	return						; yes -- return	
	decf	button_released,f	; decrement button released time counter
    btfsc	PORTA,4				; is button still released
	return 						; yes -- return
	movf	debounce,w	 
	movwf	button_released		; else set the button released debounce time counter
	return

is_sw1_pressed:
	clrw
	iorwf	button_released,w	
	btfsc	STATUS,Z			; is button released time complete 
	return						; yes -- return
	clrw
	iorwf	button_pressed,w	
	btfsc	STATUS,Z			; is button pressed time complete  
	return						; yes -- return 
	decf	button_pressed,f	; decrement button pressed time counter
	btfss	PORTA,4				; is button still pressed
	return 						; yes -- return
	movf	debounce,w			; 
	movwf	button_pressed		; else set the button pressed debounce time counter
	return

inc_counter
	clrw
	iorwf	button_released,w	
	btfss	STATUS,Z			; is button released time complete 
	return						; no -- return
	clrw
	iorwf	button_pressed,w	
	btfss	STATUS,Z			; is button pressed time complete 
	return						; no -- return 
	incf	counter,f			; increment button press counter
	movf	debounce,w	; 
	movwf	button_pressed		; set the button pressed debounce time counter
	movwf	button_released		; set the button released debounce time counter
	return

    end
