; *******************************************************************
; EDU10 Lesson 10 - Interrupts
;
; This shows how to configure and use the Timer 0 interrupt to 
; trigger reading the Analog-to-Digital every time Timer 0 overflows.
;
; *******************************************************************
; * For more details see Microchip's 44-pin Demo Board User's Guide *
; * Chapter 3. 44-Pin Demo Board Lessons                            *
; * http://ww1.microchip.com/downloads/en/DeviceDoc/41296B.pdf      *
; *******************************************************************

#include <p16F882.inc>
	__CONFIG    _CONFIG1, _LVP_OFF & _FCMEN_OFF & _IESO_OFF & _BOR_OFF & _CPD_OFF & _CP_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT
	__CONFIG    _CONFIG2, _WRT_OFF & _BOR21V

     cblock     0x20
Delay1               ; Assign an address to label Delay1
Delay2     
Display              ; define a variable to hold the diplay
Direction 
LookingFor
T0Semaphore
tmp
     endc
     
; Flag Definitions
     cblock 0x70     ; put these up in unbanked RAM
W_Save
STATUS_Save
     endc
     
     org 0
     goto      Start

     org 4
ISR:   
     movwf     W_Save              ; Save context
     movf      STATUS,w
     movwf     STATUS_Save
     
;     btfsc     PIR1,T1IF           ; Uncomment to check Timer 1 (routine needed)
;     goto      ServiceTimer1
     btfsc     INTCON,T0IF
     goto      ServiceTimer0
;     btfsc     PIC1,ADIF           ; Uncomment to check the ADC (routine needed)
;     goto      ServiceADC
     goto      ExitISR          
     
ServiceTimer0:
     bcf       STATUS,RP0          ; Ensure ISR executes in Register Bank 0
     bcf       STATUS,RP1
     bcf       INTCON,T0IF         ; clear the interrupt flag. (must be done in software)
     bsf       T0Semaphore,0       ; signal the main routine that the Timer has expired
     bsf       ADCON0,GO_DONE      ; start conversion
     btfsc     ADCON0,GO_DONE      ; this bit will change to zero when the conversion is complete
     goto      $-1
     comf      ADRESH,w            ; Form the 1's complement of ADresult
     movwf     TMR0                ; Also clears the prescaler
     goto      ExitISR
               
ExitISR:
     movf      STATUS_Save,w       ; Restore context
     movwf     STATUS
     swapf     W_Save,f            ; swapf doesn't affect Status bits, but MOVF would
     swapf     W_Save,w
     retfie
     
     
Start:
     bsf       STATUS,RP0          ; select Register Bank 1
     movlw     0xFF
     movwf     TRISA               ; Make PORTA all input
	 movlw     b'00001111'
     movwf     TRISC               ; make IO Pins RC4-RC7 output
     movlw     0x00                ; Left Justified, Vdd-Vss referenced
     movwf     ADCON1

     movlw     B'10000111'         ; configure Prescaler on Timer0, max prescale (/256)
     movwf     OPTION_REG          ; configure

     bsf       STATUS,RP1          ; select Register Bank 3
     movlw     0x07                ; we want PORTA pins 0-2 Analog
     movwf     ANSEL
     bcf       STATUS,RP0          ; back to Register Bank 0
     bcf       STATUS,RP1
     
     movlw     b'01001001'
     movwf     ADCON0              ; configure Analog-to-Digital for Fosc/8, Channel 2 (RA2), 
						           ; and turn on the Analog-to-Digital module
     movlw     0x08
     movwf     Display
     clrf      Direction
     clrf      LookingFor          ; Looking for a 0 on the button     
     
     movlw     B'10100000'         ; enable Timer 0 and global interrupts
     movwf     INTCON
MainLoop:
     btfss     T0Semaphore,0       ; did the Timer0 overflow?
     goto      CheckButton         ; no - go monitor the button
     bcf       T0Semaphore,0       ; clear the flag     
     movf      Display,w           ; Copy the display to the LEDs
     movwf     tmp
     swapf     tmp,w			   ; Swap nibbles
     movwf     PORTC			   ; show on LEDs the low nibble of counter 

     
Rotate:
     bcf       STATUS,C            ; ensure the carry bit is clear
     btfss     Direction,0
     goto      RotateLeft
RotateRight:
     rrf       Display,f
     btfsc     STATUS,C            ; Did the bit rotate into the carry?
     bsf       Display,7           ; yes, put it into bit 7.

     goto      CheckButton
RotateLeft:
     rlf       Display,f           ; rotate in place
     btfsc     STATUS,C            ; did it rotate out of the display
     bsf       Display,0           ; yes, put it into bit 0

CheckButton:
     btfsc     LookingFor,0        ; which direction are we looking for
     goto      LookingFor1
LookingFor0:
     btfss     PORTA,4             ; is the switch pressed (1)
     goto      EndMainLoop
     bsf       LookingFor,0        ; yes  Next we'll be looking for a 1
     movlw     0xFF                ; load the W register incase we need it
     xorwf     Direction,f         ; yes, flip the direction bit
     goto      EndMainLoop

LookingFor1:
     btfss     PORTA,4             ; is the switch pressed (1)
     bcf       LookingFor,0

EndMainLoop:
     movlw     .13
     call      Delay               ; delay ~10mS (13 * 775uS)
     goto      MainLoop

; Delay Subroutine.  Enter delays Wreg * 771uS + 5 uS including call and return
Delay:
     movwf     Delay2              ;
DelayLoop:
     decfsz    Delay1,f            ; Waste time.  
     goto      DelayLoop           ; The Inner loop takes 3 instructions per loop * 256 loopss = 768 instructions
     decfsz    Delay2,f            ; The outer loop takes and additional 3 instructions per lap * 256 loops
     goto      DelayLoop           ; (768+3) * 256 = 197376 instructions / 1M instructions per second = 0.197 sec.
                                   ; call it two-tenths of a second.
     return
     
     end
     
