; *******************************************************************
; EDU10 Lesson 12 - Table lookup
;
; This shows using a table lookup function to implement a 
; binary to 7-segment code conversion.  The Pot is read by the Analog-to-Digital converter, 
; then the high order 4 bits are converted to 7-segment code and
; displayed on 7-segment display.
;
; *******************************************************************
; * For more details see Microchip's 44-pin Demo Board User's Guide *
; * Chapter 3. 44-Pin Demo Board Lessons                            *
; * http://ww1.microchip.com/downloads/en/DeviceDoc/41296B.pdf      *
; *******************************************************************

#include <p16F882.inc>
	__CONFIG    _CONFIG1, _LVP_OFF & _FCMEN_OFF & _IESO_OFF & _BOR_OFF & _CPD_OFF & _CP_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT
	__CONFIG    _CONFIG2, _WRT_OFF & _BOR21V


     cblock 0x20
temp
     endc

     org 0     
Start:
     bsf       STATUS,RP0          ; select Register Bank 1
     movlw     0xFF
     movwf     TRISA               ; Make PORTA all input
     clrf      TRISB               ; Make PORTB all output 
     movlw     0x00                ; Left Justified, Vdd-Vss referenced
     movwf     ADCON1
     bsf       STATUS,RP1          ; select Register Bank 3
     movlw     0x07                ; we want PORTA pins 0-2 Analog
     movwf     ANSEL
     bcf       STATUS,RP0          ; back to Register Bank 0
     bcf       STATUS,RP1     
     movlw     b'01001001'
     movwf     ADCON0              ; configure Analog-to-Digital for Fosc/8, Channel 2 (RA2), 
						           ; and turn on the Analog-to-Digital module
MainLoop:
     nop                           ; wait 5uS for Analog-to-Digital amp to settle and capacitor to charge.
     nop                           ; wait 1uS
     nop                           ; wait 1uS
     nop                           ; wait 1uS 
     nop                           ; wait 1uS
     bsf       ADCON0,GO_DONE      ; start A2D conversion
     btfsc     ADCON0,GO_DONE      ; wait until the conversion is complete
     goto      $-1

     swapf     ADRESH,w            ; read the A2D, move the high nibble to the low part
     call      BinaryTo7Segment    ; Convert to 7-Segment Code
     movwf     PORTB               ; on Port B
     goto      MainLoop
     
          
; Convert 4 bit binary to 4 bit 7-segment code
; 
     org     0xf7                  ; force table to cross a 256 instruction boundary
BinaryTo7Segment:
     andlw     0x0F                ; mask off invalid entries
     movwf     temp
     movlw     high TableStart     ; get high order part of the beginning of the table
     movwf     PCLATH
     movlw     low TableStart      ; load starting address of table
     addwf     temp,w              ; add offset
     btfsc     STATUS,C            ; did it overflow?
     incf      PCLATH,f            ; yes: increment PCLATH
     movwf     PCL                 ; modify PCL

TableStart:
	retlw	B'00111111' ;decode 0
	retlw	B'00000110' ;decode 1
	retlw	B'01011011' ;decode 2
	retlw	B'01001111' ;decode 3
	retlw	B'01100110' ;decode 4
	retlw	B'01101101' ;decode 5
	retlw	B'01111101' ;decode 6
	retlw	B'00000111' ;decode 7
	retlw	B'01111111' ;decode 8
	retlw	B'01101111' ;decode 9
	retlw	B'10111111' ;decode 0.
	retlw	B'10000110' ;decode 1.
	retlw	B'11011011' ;decode 2.
	retlw	B'11001111' ;decode 3.
	retlw	B'11100110' ;decode 4.
	retlw	B'11101101' ;decode 5.
	retlw	B'11111101' ;decode 6.
	retlw	B'10000111' ;decode 7.
	retlw	B'11111111' ;decode 8.
	retlw	B'11101111' ;decode 9.

     end
